/* _MiscTreeBrowserNode
 *
 */

// IMPORTS

#import <libc.h>
#import <ansi/assert.h>

#import <objc/List.h>
#import <objc/hashtable.h>

#import <misckit/_MiscTreeBrowserNode.h>


// IMPLEMENTATION

@implementation _MiscTreeBrowserNode

- initWithString:(const char *)aString parent:anObject
{
	[super init];

	[self setStringValue:aString];
	[self setParent:anObject];

	return self;
}

- init
{
	return [self initWithString:NULL parent:nil];
}

- free
{
	if (stringValue) free(stringValue);
	stringValue = NULL;

	children = [[children freeObjects] free];

	return [super free];
}

- (BOOL)canFree
{
	return YES;
}

- setParent:(id<MiscTreeBrowserNodes>)aNode
{
	if (parent != aNode)
	{
		parent = aNode;
		[parent addChild:self];
	}

	return self;
}

// MiscTreeBrowserNodes methods

- (const char *)stringValue
{
	return stringValue;
}

- parent
{
	return parent;
}

- (unsigned int)childCount
{
	return children ? [children count] : 0;
}

- childAt:(int)index
{
	return children ? [children objectAt:index] : nil;
}

- (int)indexOfChild:(id<MiscTreeBrowserNodes>)aNode
{
	return children ? [children indexOf:aNode] : NX_NOT_IN_LIST;
}

// MiscTreeBrowserEditing methods

- setStringValue:(const char *)aString
{
	if (stringValue)
	{
		free(stringValue);
	}
	stringValue = aString ? NXCopyStringBuffer(aString) : NULL;

	return self;
}

// MiscTreeBrowserModifications methods

- createChildAfter:(id<MiscTreeBrowserNodes>)aNode
/*"
Returns: child/nil.
"*/
{
	assert([aNode parent] == self);
	{
		id kid = [[[self class] alloc] init];
		int index = [self indexOfChild:aNode];

		if ((index != NX_NOT_IN_LIST) && [self addChild:kid at:(index + 1)])
		{
			return kid;
		}
		else
		{
			[kid free];
		}
	}

	return nil;
}

- createSibling
{
	id pa = [self parent];

	return pa ? [pa createChildAfter:self] : [[[self class] alloc] init];
}

- addChild:(id<MiscTreeBrowserNodes>)aNode at:(int)index
{
	id pa = [aNode parent];

	assert((pa == self) || (pa == nil));

	if (children)
	{
		if ([children indexOf:aNode] == NX_NOT_IN_LIST)
		{
			[children insertObject:aNode at:index];
		}
	}
	else
	{
		children = [[List alloc] init];
		[children addObject:aNode];
	}

	if (pa != self)
	{
		[aNode setParent:self];
	}

	return self;
}

- addChild:(id<MiscTreeBrowserNodes>)aNode
{
	id pa = [aNode parent];

	assert((pa == self) || (pa == nil));

	return [self addChild:aNode at:[self childCount]];
}


- removeChildAt:(int)index
{
	if (children && (index >= 0) && (index < [self childCount]))
	{
		id node = [children objectAt:index];

		if ([node canFree])
		{
			[node setParent:nil];
			[children removeObjectAt:index];
			[node free];

			return nil;			// success
		}
	}

	return self;				// failure, return non-zero value
}

- removeChild:(id<MiscTreeBrowserNodes>)aNode
{
	assert([aNode parent] == self);

	if (children)
	{
		unsigned int index = [children indexOf:aNode];

		if (index != NX_NOT_IN_LIST)
		{
			return [self removeChildAt:index];
		}
	}

	return self;				// failure, return non-zero value
}

@end
